package Crawfis680.Grid;

import java.security.InvalidParameterException;
import java.util.Iterator;

import Crawfis680.IGraph.IGraph;

public class Grid<N,E> implements IGraph<N,E> {
	//
	// IGraph Interface
	//
	@Override
	public Boolean HasEdge(int fromNodeIndex, int toNodeIndex) {
		int fromRow = getRow(fromNodeIndex);
		int fromColumn = getColumn(fromNodeIndex);
		int toRow = getRow(toNodeIndex);
		int toColumn = getColumn(toNodeIndex);
		return HasEdge(fromRow, fromColumn, toRow, toColumn);
	}

	@Override
	public int NumberOfEdges() {
		// Count each node's South and West edges only (North and East will be counted
		// by the adjacent nodes).
		return 2*((numberRows-1)*(numberColumns-1)) + (numberRows-1) + (numberColumns-1);
	}

	@Override
	public int NumberOfNodes() {
		return numberRows * numberColumns;
	}

	@Override
	public Iterable<Integer> getAdjacentNodes(int nodeIndex) {
		return new NodeIterable(nodeIndex);
	}
	
	
	// Bad Design #2
	// TODO Fix Bad return value
	@Override
	public E getEdgeLabel(int fromNodeIndex, int toNodeIndex) {
		return null;
	}

	@Override
	public N getNodelLabel(int index) throws InvalidParameterException {
		int row = getRow(index);
		int column = getColumn(index);
		return getNodelLabel(row,column);
	}
	
	
	
	//
	// Specific Public Methods for Grid.
	//
	// Bad Design #3
	// TODO Fix bad constructor
	public Grid(IGridDataAccessor<N> nodeAccessor) {
		this.nodeAccessor = nodeAccessor;
		this.numberRows = nodeAccessor.Height();
		this.numberColumns = nodeAccessor.Width();
	}
	
	public Boolean HasEdge(int fromRow, int fromColumn, int toRow, int toColumn) {
		// Check the row and column indices are within the grid.
		if( !ValidIndex(fromRow, fromColumn))
			return false;
		if( !ValidIndex(toRow, toColumn))
			return false;
		
		// Compare the row and column indices to see if this is a valid edge for a grid.
		int rowDifference = Math.abs(fromRow - toRow);
		int columnDifference = Math.abs(fromColumn - toColumn);
		if( rowDifference > 1)
			return false;
		if( columnDifference > 1)
			return false;
		// Pretty simple now that we know a) both nodes are valid and 
		// b) all indices are within one from each other. Now we just
		// need to make sure that we do not have a diagonal.
		if(rowDifference == columnDifference)
			return false;
		// Alternatively, the last three if statements can be combined as follows:
		//if( (rowDifference+columnDifference) != 1)
		//	return false;
		return true;
	}
	
	public N getNodelLabel(int row, int column) throws InvalidParameterException {
		if(!ValidIndex(row,column))
			throw new InvalidParameterException();
		return this.nodeAccessor.getNodeLabel(row,column);
	}

	//
	// The next three methods could have private access or package access
	//
	public int getColumn(int nodeIndex) {
		return nodeIndex % numberColumns;
	}

	public int getRow(int nodeIndex) {
		return nodeIndex / numberColumns;
	}
	
	public int getIndex(int row, int column) {
		return row*numberColumns + column;
	}
	

	//
	// Implementation
	//
	private Boolean ValidIndex( int row, int column ) {
		return ValidRow(row) && ValidColumn(column);
	}
	
	private Boolean ValidRow( int row ) {
		if( row < 0)
			return false;
		if(row >= numberRows)
			return false;
		return true;
	}

	private Boolean ValidColumn(int column) {
		if(column < 0)
			return false;
		if(column >= numberColumns)
			return false;
		return true;
	}

	//
	// Private member variables
	//
	private IGridDataAccessor<N> nodeAccessor;
	private int numberRows;
	private int numberColumns;


	// Kind of annoying I need to declare these constants here. I want them
	// only in NodeIterator :-(.
	private static final int[] rowStep = { 0, 1, 0, -1 };
	private static final int[] columnStep = { -1, 0, 1, 0 };
	
	
	//
	// Nested supporting classes
	//
	class NodeIterable implements Iterable<Integer> {

		@Override
		public Iterator<Integer> iterator() {
			return new NodeIterator(nodeIndex);
		}
		
		NodeIterable(int nodeIndex) {
			this.nodeIndex = nodeIndex;
		}
		
		private int nodeIndex;
		
		class NodeIterator implements Iterator<Integer> {
			@Override
			public boolean hasNext() {
				return currentIndex < 4;
			}

			@Override
			public Integer next() {
				int index = getIndex(row+rowStep[currentIndex], column+columnStep[currentIndex]);
				currentIndex++;
				MoveNext();
				return index;
			}

			@Override
			public void remove() {				
				throw new UnsupportedOperationException("The method remove() is not allowed with this iterator.");
			}
			

			NodeIterator(int nodeIndex) {
				this.row = getRow(nodeIndex);
				this.column = getColumn(nodeIndex);
				Initialize();
			}
			
			private void Initialize() {
				if(!ValidIndex(row, column))
					currentIndex = 4;
				MoveNext();			
			}

			private void MoveNext() {
				while( currentIndex < 4 && !ValidIndex(row+rowStep[currentIndex], column+columnStep[currentIndex]))
					currentIndex++;
			}
			
			private int row, column;
			private int currentIndex = 0;
		}
	}
}
