
package Crawfis680.IGraph;

public interface IGraph<N, E> {
	//
	// Methods dealing with the overall topology
	//
	public int NumberOfNodes();
	public int NumberOfEdges();
	public Boolean HasEdge(int fromNodeIndex, int toNodeIndex);
	//
	// Design Problem - Java Generics have limitations:
	// Sadly, Java decided not to update the JVM when it added
	// generics, so all generics need to be of reference type.
	// Hence an array might be a better return type than a
	// List if performance is key. Alternatively, we could
	// make our own iterator type specific to integers. This
	// would not follow Java's Iterable Iterator interfaces. Why
	// this over arrays? Arrays may lead to confusion. Can I set
	// the value of an index. If so, what happens?
	//
	public Iterable<Integer> getAdjacentNodes(int nodeIndex);
	//public int[] getAdjacentNodes(int nodeIndex);
	//public MyIntegerInterator getAdjacentNodes(int nodeIndex);
	//
	// Methods dealing with Node labels.
	//
	// This could be factored out into its own interface
	// Alternatively, implementers without nodal values can just 
	// return a default value. Possible in C# for all types - 
	// primitive and reference types. Java can return null, since
	// all generics are reference types only. Separate classes are 
	// required for value types :-(. Backward compatibility.
	//
	public N getNodelLabel(int nodeIndex);
	//
	// Methods dealing with Edge labels.
	//
	// This could be factored out into its own interface as above.
	// However, bifurcating node and edge labels into two separate 
	// interfaces is problematic for the using class as they need 
	// to deal with two different types rather than a single polymorphic 
	// type.
	//
	public E getEdgeLabel(int fromNodeIndex, int toNodeIndex);
}
